package sample;
import org.capcaval.lafabrique.application.ApplicationTools;
import org.capcaval.lafabrique.application.annotations.AppInformation;
import org.capcaval.lafabrique.application.annotations.AppProperty;
import org.capcaval.lafabrique.commandline.Command;
import org.capcaval.lafabrique.commandline.CommandParam;
import org.capcaval.lafabrique.common.CommandResult;

@AppInformation (
		about= {"It shows a sample of application with several type of commands."})

public class Sample{
	public enum Country{ France, GreatBritain, USA};
	public enum City{ Paris, London, NewYork};
	@AppProperty(comment = "Keep persitent the greeted name.", persistence = true)
	private String name;

	public static void main(String[] args) {
		CommandResult r = ApplicationTools.runApplication(Sample.class, args);
		System.out.println(r.toString());
	}
	
	// laFab convert the parameter and also the return value
	// it does handle primitive and basic classes.
	@Command(desc="add two number")
	public Integer add(@CommandParam(name="firstNumber", desc="First Number")int a, 
			@CommandParam(name="secondNumber", desc="Secondnumber Number")int b){
		return a+b;
	}

	// laFab does also handle multiple parameters
	@Command(desc="multiply several numbers")
	public int multiply(@CommandParam(name="numberArray", desc="Array of number")int... numberArray){
		// beware to initialize with 1 and not zero
		// otherwise the result shall always be zero.
		int returnedValue = 1;
		
		for(int val : numberArray){
			returnedValue = returnedValue * val;
		}
		
		return returnedValue;
	}

	// laFab does also handle array
	@Command(desc="add an array of number")
	public Integer addMulti(@CommandParam(name="numberArray", desc="Array of number to multiply.")int[] numberArray){
		int returnedValue = 0;
		
		for(int val : numberArray){
			returnedValue = returnedValue + val;
		}
		return returnedValue;
	}

	// laFab does also handle enum.
	@Command(desc="Get the country of a bug city. Ex of use: ./laFabSample City.London")
	public Country getCountryOfCity(@CommandParam(name="city", desc="Enum value of City")City city){
		Country country = Country.France;
		
		if(city == City.London){
			country = Country.GreatBritain;
		}
		else if(city == City.NewYork){
			country = Country.USA;
		}

		return country;
	}


	// laFab provides the use of properties
	@Command(desc="Keep persitent and Greet the given name")
	public String greet(@CommandParam(name="name", desc="Name to be greeted")String name){
		this.name = name;
		return "Hello " + this.name + "!";
	}
	
	
	// laFab provides the use of persistent properties
	@Command(desc="Greet with the last persitent name")
	public String greet(){
		return "Hello " + this.name + "!";
	}
}
